"""structures and classes for storing hdf5 data

see https://portal.hdfgroup.org/display/HDF5/Introduction+to+HDF5

note: these structures are intentionally made independent of h5py library as this h5py library is not available in all python environments. More specifically, h5py is not available in jython. So the purpose of this library is to allow storage of hdf5 data in all environments.
"""

class ElementType(object):
    U1=0
    U8=1
    U32=2
    S32=3
    F32=4
    F64=5

class DataSet(object):
    """ an hdf5 dataset    
    """
    def __init__(self, name, size, element_type=ElementType.U32):
        """
        :param str name: the name of the dataset
        :param tuple(int) size: the size of each dimension of the dataset
        """
        self.name = name
        self.size = size
        self._element_type = element_type
        self._elements = []
        for i in range(self.num_elements):  # pylint: disable=unused-variable
            self._elements.append(0)

    @property
    def element_type(self):
        return self._element_type

    @property
    def dimension(self):
        return len(self.size)

    @property
    def elements(self):
        return self._elements

    @property
    def num_elements(self):
        num_elements = 1
        for i in range(self.dimension):
            num_elements *= self.size[i]
        return num_elements

    def get_element(self, pos):
        assert len(pos) == len(self.size)
        return self._elements[self._pos_to_index(pos)]

    def __getitem__(self, pos):
        assert len(pos) == len(self.size)
        return self._elements[self._pos_to_index(pos)]

    def set_element(self, pos, value):
        assert len(pos) == len(self.size)
        self._elements[self._pos_to_index(pos)] = value

    def __setitem__(self, pos, value):
        assert len(pos) == len(self.size)
        self._elements[self._pos_to_index(pos)] = value

    def _pos_to_index(self, pos):
        index = 0
        for i in range(self.dimension):
            index += pos[i]
            if i < self.dimension - 1:
                index *= self.size[i]
        return index


class Group(object):

    def __init__(self, name):
        """
        :param str name: the name of the group
        """
        self.name = name
        self._datasets = {}
        self._groups = {}

    def add_dataset(self, dataset):
        assert isinstance(dataset, DataSet)
        self._datasets[dataset.name] = dataset

    def add_group(self, group):
        assert isinstance(group, Group)
        self._datasets[group.name] = group

    @property
    def datasets(self):
        return self._datasets

    @property
    def groups(self):
        return self._groups

    def __getitem__(self, dataset_name):
        return self._datasets[dataset_name]

